import click
import logging

from certsGenerator.certManager import CertManager


class CertsGenerator:
    def __init__(self, pathToConf: str, debug: bool):
        self.CONF_FILE = pathToConf
        logging_format = "%(levelname)s - %(message)s"
        if not debug:
            logging.basicConfig(level=logging.INFO, format=logging_format)
        else:
            logging.basicConfig(level=logging.DEBUG, format=logging_format)

    def run(self) -> None:
        cerManager = CertManager(confFile=self.CONF_FILE)
        for certConf in cerManager.conf.general["certs"]:
            cerManager.createCerts(certName=certConf["name"])


@click.command()
@click.option("--conf", "-c", default="data/conf.json", help="Certs configuration file")
@click.option(
    "--debug",
    "-d",
    default="false",
    help="Debug mode: true or false",
    type=click.Choice(["true", "false"]),
)
def cli(conf: str, debug: str) -> None:
    if debug == "false":
        CertsGenerator(pathToConf=conf, debug=False).run()
    else:
        CertsGenerator(pathToConf=conf, debug=True).run()


if __name__ == "__main__":
    cli()



import sys
import logging
from os.path import exists


def loadFile(fileName: str) -> bytes:
    if exists(fileName):
        content = b""
        try:
            with open(fileName, mode="rb") as f:
                content = f.read()
        except OSError as e:
            sys.exit(f"failed to open {fileName}: {e}")

        if len(content) == 0:
            logging.error(f"File {fileName} is empty")
            raise ValueError()
            sys.exit()

        logging.debug(f"File {fileName} loaded with size of {len(content)} bytes")
        return content  # type: ignore
    else:
        logging.error(f"File {fileName} doesn't exist")
        raise ValueError()
        sys.exit()



import logging
import os
import stat
import sys
from typing import Tuple, Union, Dict, Optional

from cryptography import x509
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import ec
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives.asymmetric import ed25519

from certsGenerator.conf import Conf
from certsGenerator.helpers import loadFile
from certsGenerator.certBuilder import CertBuilder


class CertManager:
    def __init__(self, confFile: str):
        self.conf: Conf = Conf(confFile=confFile)

    def storePrivateKey(
        self,
        certName: str,
        private_key: Union[
            ec.EllipticCurvePrivateKey, rsa.RSAPrivateKey, ed25519.Ed25519PrivateKey
        ],
    ) -> None:
        certConf = self.conf.getCert(certName=certName)

        # get passphrase
        passphrase = self.conf.getPassphrase(certName=certName)

        # set encryption algorithm
        encryption_algorithm = self._getEncryptionAlgorithm(passphrase=passphrase)

        # get other params from conf
        encoding, key_format = self._getParamsForPrivateBytes(certConf=certConf)

        # get path and save the key file
        path = self.conf.getCertPath(certName=certName, ext="private_key")
        try:
            with open(path, mode="wb") as f:
                f.write(
                    private_key.private_bytes(  # type: ignore
                        encoding=encoding,
                        format=key_format,
                        encryption_algorithm=encryption_algorithm,
                    )
                )
            try:
                os.chmod(path, mode=0o600)
            except OSError as e:
                logging.error(f"can't set permission {stat.S_IRUSR} on {path}: {e}")
                sys.exit()
        except OSError as e:
            logging.error(f"failed to write file {path}: {e}")
            sys.exit()

    def storePublicKey(self, certName: str, cert: x509.Certificate) -> None:
        path = self.conf.getCertPath(certName=certName, ext="signed_certificate")
        certConf = self.conf.getCert(certName=certName)
        # get other params from conf
        encoding, format = self._getParamsForPublicBytes(certConf=certConf)
        try:
            if certConf.get("public_key").get("encoding") == "OpenSSH":
                print(type(cert.public_key()))
                content = cert.public_key().public_bytes(encoding, format)
                with open(path, "wb") as f:
                    f.write(content)
            # DER or PEM encodings
            else:
                content = cert.public_bytes(encoding)
                with open(path, "wb") as f:
                    f.write(content)

        except OSError:
            logging.error(f"can't save public key in {path}")
            sys.exit()

    def getPrivateKey(
        self, certName: str
    ) -> Union[ec.EllipticCurvePrivateKey, rsa.RSAPrivateKey]:
        certConf = self.conf.getCert(certName=certName)
        if "storage" not in certConf.keys():
            raise ValueError(f"key not found in {certConf}")
            sys.exit()
        path = certConf["storage"]["path"]
        certName = certConf["subject_name"]

        keyFile = self.conf.getCertPath(certName=certName, ext="private_key")

        private_key = None
        if not os.path.exists(path):
            os.makedirs(path)

        if os.path.exists(keyFile):
            logging.info(f"Private key of {certName} file already exist")
            private_key_bytes = loadFile(fileName=keyFile)
            password = self.conf.getPassphrase(certName=certName)
            try:
                enc = certConf.get("private_key").get("encoding")
                if enc == "PEM":
                    private_key = serialization.load_pem_private_key(  # type: ignore
                        private_key_bytes, password=password
                    )
                elif enc == "DER":
                    private_key = serialization.load_der_private_key(  # type: ignore
                        private_key_bytes, password=password
                    )
                elif enc == "OpenSSH":
                    private_key = serialization.load_ssh_private_key(  # type: ignore
                        private_key_bytes, password=password
                    )
            except ValueError as e:
                logging.error(
                    f"Error while loading the private key {keyFile}. Please make sure the key is properly generated and the file is not empty. {e}"
                )
                sys.exit()

        else:
            logging.info(f"Creating private key of {certName}")
            key_type: str = certConf["private_key"]["algorithm"]["type"].upper()
            params = []

            if certConf.get("private_key").get("algorithm").get("params"):
                params = certConf["private_key"]["algorithm"]["params"]
            else:
                params = None

            if key_type == "EC":
                private_key = ec.generate_private_key(  # type: ignore
                    curve=self.conf.curveMapping[params["curve"]]
                )
            elif key_type == "RSA":
                private_key = rsa.generate_private_key(  # type: ignore
                    public_exponent=65537, key_size=int(params["key_size"])
                )
            elif key_type == "ED25519":
                private_key = ed25519.Ed25519PrivateKey.generate()
            else:
                logging.error(f"Key type not found or implemented {key_type}")
                raise ValueError()
                sys.exit()

            self.storePrivateKey(
                certName=certName, private_key=private_key
            )  # type: ignore
        return private_key  # type: ignore

    def createCerts(self, certName: str) -> None:
        certConf = self.conf.getCert(certName=certName)
        # get the private key (created if it doesn't exist yet)
        private_key = self.getPrivateKey(certName=certName)

        # regarding cert
        certFile = self.conf.getCertPath(certName=certName, ext="signed_certificate")
        issuer_name = certConf["issuer_name"]
        # check if cert exists, otherwise create it
        if not os.path.exists(certFile):
            logging.info(f"Creating cert file for {certName}...")
            # but to create the cert, we need issuer key to sign it
            issuer_private_key = self.getPrivateKey(certName=issuer_name)
            # ok then we build the cert
            subject_name = certConf["subject_name"]
            ski = x509.SubjectKeyIdentifier.from_public_key(private_key.public_key())
            cert = CertBuilder(certName=subject_name, conf=self.conf, ski=ski).builder
            cert = cert.public_key(private_key.public_key())
            # now sign the cert with the issuer private key
            hashAlg = self.conf.hashMapping[certConf["private_key"]["sign_with_alg"]]
            cert = cert.sign(
                private_key=issuer_private_key, algorithm=hashAlg
            )  # type: ignore
            # now store the cert
            self.storePublicKey(certName=certName, cert=cert)  # type: ignore
            logging.info(
                f"Cert created for {certName} with crt signed by {issuer_name}"
            )
        else:
            logging.info(f"Cert file already exists for {certName}")

        self._checkSignature(subjectName=certName, issuerName=issuer_name)

    def _checkSignature(self, subjectName: str, issuerName: str) -> None:
        # conf
        logging.info(f"checking signature of {subjectName}")
        issuerCrtFile = self.conf.getCertPath(
            certName=issuerName, ext="signed_certificate"
        )
        subjectCrtFile = self.conf.getCertPath(
            certName=subjectName, ext="signed_certificate"
        )

        # load certificates files
        issuer_public_key = b""
        subject_public_key = b""
        if issuerCrtFile != subjectCrtFile:
            issuer_public_key = loadFile(fileName=issuerCrtFile)
            subject_public_key = loadFile(fileName=subjectCrtFile)
        else:
            subject_public_key = loadFile(fileName=subjectCrtFile)
            issuer_public_key = subject_public_key

        # extract issuer and subject public keys
        issuer_cert_conf = self.conf.getCert(certName=issuerName)
        subject_cert_conf = self.conf.getCert(certName=subjectName)
        issuer_enc = issuer_cert_conf.get("public_key").get("encoding")
        subj_enc = subject_cert_conf.get("public_key").get("encoding")
        cert_to_check = ""

        if issuer_enc == "PEM":
            issuer_public_key = x509.load_pem_x509_certificate(
                issuer_public_key
            ).public_key()
        elif issuer_enc == "DER":
            issuer_public_key = x509.load_der_x509_certificate(
                issuer_public_key
            ).public_key()

        if subj_enc == "PEM":
            cert_to_check = x509.load_pem_x509_certificate(subject_public_key)
        if subj_enc == "DER":
            cert_to_check = x509.load_der_x509_certificate(subject_public_key)

        # checking now
        if isinstance(issuer_public_key, rsa.RSAPublicKey):
            issuer_public_key.verify(
                cert_to_check.signature,
                cert_to_check.tbs_certificate_bytes,
                self.conf.RSApaddingMapping["PKCS1v15"](),
                cert_to_check.signature_hash_algorithm,
            )
        elif isinstance(issuer_public_key, ec.EllipticCurvePublicKey):
            issuer_public_key.verify(  # type: ignore
                cert_to_check.signature,
                cert_to_check.tbs_certificate_bytes,
                ec.ECDSA(cert_to_check.signature_hash_algorithm),
            )
        elif isinstance(issuer_public_key, ed25519.Ed25519PublicKey):
            issuer_public_key.verify(  # type: ignore
                cert_to_check.signature,
                cert_to_check.tbs_certificate_bytes,
            )
        else:
            logging.error(
                f"Failed to verify due to unsupported algorythm {type(cert_to_check.public_key())}"
            )
            raise ValueError()
            sys.exit()

        logging.info(f"signature validated for {subjectName}")

    def _getParamsForPrivateBytes(
        self, certConf: Dict
    ) -> Tuple[serialization.Encoding, serialization.PrivateFormat]:
        encoding = self.conf.encodingMapping.get(
            certConf.get("private_key").get("encoding")
        )
        key_format = self.conf.privateSerializationMapping[
            certConf.get("private_key").get("serialization")
        ]

        return encoding, key_format

    def _getParamsForPublicBytes(
        self, certConf: Dict
    ) -> Tuple[serialization.Encoding, serialization.PublicFormat]:
        encoding = self.conf.encodingMapping.get(
            certConf.get("public_key").get("encoding")
        )
        key_format: serialization.PublicFormat = None
        try:
            # default key format is SubjectPublicKeyInfo if not specified
            if certConf.get("public_key").get("serialization", None) is None:
                key_format = self.conf.publicSerializationMapping[
                    "SubjectPublicKeyInfo"
                ]
            else:
                key_format = self.conf.publicSerializationMapping[
                    certConf.get("public_key").get("serialization")
                ]
        except ValueError as e:
            logging.error(f"Enable to set key format due to error {e}")
            sys.exit()

        return encoding, key_format

    def _getEncryptionAlgorithm(
        self, passphrase: Optional[str] = None
    ) -> Union[serialization.BestAvailableEncryption, serialization.NoEncryption]:
        if passphrase:
            encryption_algorithm = serialization.BestAvailableEncryption(passphrase)
        else:
            encryption_algorithm = serialization.NoEncryption()

        return encryption_algorithm



import datetime
import sys
import logging

from orjson import loads, JSONEncodeError
from typing import Union

from cryptography import x509
from cryptography.x509.oid import NameOID
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import ec
from cryptography.hazmat.primitives.asymmetric import padding

from certsGenerator.helpers import loadFile


checkRegistry = []


class Conf:
    def __init__(self, confFile: str):
        self.general = self._load(confFile)
        self.fileExtenstions = self._getFileExt()
        self.checkRegistry = checkRegistry

        # run checks
        self._checkCertName()
        self._checkEncoding()
        self._checkExtendedKeyUsage()
        self._checkKeyUsage()
        self._checkNotValidDate()
        self._checkPubKeyEncodingPresent()
        self._checkRedundantNames()
        self._checkSerialization()

    def getCert(self, certName: str) -> dict:
        certs = self.general["certs"]
        conf = {}
        for cert in certs:
            if cert["name"] == certName:
                conf = cert
                break
        if not (len(conf) > 0):
            logging.error(f"cert {certName} not found")
            raise ValueError()
            sys.exit()
        return conf["conf"]

    def get(self, certName: str, field: str, isExt: bool = True) -> dict:
        certConf = self.getCert(certName)
        if isExt is True:
            return certConf["extensions"][field]
        elif certConf[field]:
            return certConf[field]
        else:
            logging.error(f"{field} not found in conf")
            raise ValueError()
            sys.exit()

    def getCertPath(self, certName: str, ext: str) -> str:
        certConf = self.getCert(certName)
        path = certConf["storage"]["path"]
        fileName = certConf["storage"]["fileName"]
        ext = self.fileExtenstions[ext]
        certpath = f"{path}/{fileName}.{ext}"
        return certpath

    def getPath(self, certName: str) -> str:
        certConf = self.getCert(certName)
        path = certConf["storage"]["path"]
        return path

    def getPassphrase(self, certName: str) -> Union[bytes, None]:
        # get passphrase
        certConf = self.getCert(certName)
        if certConf.get("private_key") and certConf.get("private_key").get("passphrase"):  # type: ignore
            p = certConf["private_key"]["passphrase"]["path"]
            n = certConf["private_key"]["passphrase"]["fileName"]
            passFile = f"{p}/{n}"
            passphrase = loadFile(passFile)

            if not type(passphrase) == bytes:
                logging.error("passphrase should be of bytes type")
                raise ValueError()
                sys.exit()
            return passphrase
        else:
            passphrase = None  # type: ignore
            return passphrase

    def _load(self, fileName: str) -> dict:
        generalConf = {}
        try:
            with open(fileName, mode="r", encoding="utf-8") as file:
                try:
                    generalConf = loads(file.read())
                except JSONEncodeError as e:
                    sys.exit(f"enable to load string from {fileName}: {e}")
        except OSError as e:
            logging.error(f"failed to open {fileName}: {e}")
        return generalConf

    def _getFileExt(self) -> dict:
        return self.general["defaults"]["file_extentions"]

    def _checkPubKeyEncodingPresent(self) -> None:
        # check if the public key encoding is persent
        certs = self.general["certs"]
        for cert in certs:
            try:
                cert.get("conf").get("public_key").get("encoding")
            except KeyError as e:
                certName = cert.get("name")
                logging.error(
                    f"Configuration file error: can't find public key encoding for {certName}, {e}"
                )
                raise ValueError()
                sys.exit()

    def _checkRedundantNames(self) -> None:
        # check if no redundant certs names
        certs = self.general["certs"]
        names: dict = {}
        for cert in certs:
            if cert["name"] in names.keys():
                names[cert["name"]] += 1
            else:
                names[cert["name"]] = 1
        for k, v in names.items():
            if v > 1:
                logging.error(f"Configuration file error: {k} appears {v} times")
                raise ValueError()
                sys.exit()

    def _checkNotValidDate(self) -> None:
        # check not_valid_before not_valid_after
        certs = self.general["certs"]
        for certConf in certs:
            certConf = certConf["conf"]
            if certConf["not_valid_before"] == "now":
                nvb = datetime.datetime.utcnow()
            elif isinstance(int(), certConf["not_valid_before"]) or isinstance(
                int, certConf["not_valid_after"]
            ):
                nvb = datetime.datetime.utcnow() + datetime.timedelta(
                    days=certConf["not_valid_before"]
                )
            else:
                logging.error(f'invalid value from {nvb}, should be of int or "now"')
                raise ValueError()
                sys.exit()

    def _checkCertName(self) -> None:
        certs = self.general["certs"]
        for cert in certs:
            subjectName = cert["conf"]["subject_name"]
            certName = cert["name"]
            if certName != subjectName:
                logging.error(
                    f"certname {certName} has to be the same than the subject name, found {subjectName}"
                )
                raise ValueError()
                sys.exit()

    def _checkKeyUsage(self) -> None:
        certs = self.general["certs"]
        for cert in certs:
            certName = cert.get("name")
            try:
                for ku in cert["conf"]["extensions"]["KeyUsage"]["items"]:
                    if ku.lower() not in self.keyUsage:
                        logging.error(
                            f"{ku} not found in allowed keyUsage for {certName}"
                        )
                        raise ValueError()
                        sys.exit()
            except Exception:
                logging.error(f"can't find KeyUsages params for {certName}")

    def _checkExtendedKeyUsage(self) -> None:
        certs = self.general["certs"]
        for cert in certs:
            if cert.get("conf").get("extensions").get("ExtendedKeyUsage"):
                for ku in (
                    cert.get("conf")
                    .get("extensions")
                    .get("ExtendedKeyUsage")
                    .get("items")
                ):
                    if ku.upper() not in self.extendedKeyUsageMapping.keys():
                        logging.error(f"{ku} not found in allowed ExtendedKeyUsage")
                        raise ValueError()
                        sys.exit()

    def _checkEncoding(self) -> None:
        certs = self.general["certs"]
        for cert in certs:
            certName = cert.get("name")
            enc_priv = cert.get("conf").get("private_key").get("encoding")
            enc_pub = cert.get("conf").get("public_key").get("encoding")
            if (enc_priv not in self.encodingMapping.keys()) or (
                enc_pub not in self.encodingMapping.keys()
            ):
                logging.error(
                    f"encoding not found for {certName} in allowed encoding formats"
                )
                raise ValueError()
                sys.exit()

    def _checkSerialization(self) -> None:
        certs = self.general["certs"]
        for cert in certs:
            certName = cert.get("name")
            serialization = cert.get("conf").get("private_key").get("serialization")
            if serialization not in self.privateSerializationMapping.keys():
                logging.error(
                    f"serialization not found for private key of {certName} in allowed serialization formats"
                )
                raise ValueError()
                sys.exit()

    nameAttributesMapping = {
        "COUNTRY_NAME": NameOID.COUNTRY_NAME,
        "STATE_OR_PROVINCE_NAME": NameOID.STATE_OR_PROVINCE_NAME,
        "LOCALITY_NAME": NameOID.LOCALITY_NAME,
        "ORGANIZATION_NAME": NameOID.ORGANIZATION_NAME,
        "COMMON_NAME": NameOID.COMMON_NAME,
    }

    extensionMapping = {
        "SubjectAlternativeName": x509.SubjectAlternativeName,
        "KeyUsage": x509.KeyUsage,
        "BasicConstraints": x509.BasicConstraints,
        "NameConstraints": x509.NameConstraints,
        "NameAttribute": x509.NameAttribute,
        "AuthorityKeyIdentifier": x509.AuthorityKeyIdentifier,
        "DNSName": x509.DNSName,
        "IPAddress": x509.IPAddress,
    }

    curveMapping = {
        "SECP256R1": ec.SECP256R1(),
        "SECP384R1": ec.SECP384R1(),
        "SECP521R1": ec.SECP521R1(),
    }

    encodingMapping = {
        "PEM": serialization.Encoding.PEM,
        "DER": serialization.Encoding.DER,
        "Raw": serialization.Encoding.Raw,
        "OpenSSH": serialization.Encoding.OpenSSH,
    }

    privateSerializationMapping = {
        "PKCS8": serialization.PrivateFormat.PKCS8,
        "TraditionalOpenSSL": serialization.PrivateFormat.TraditionalOpenSSL,
        "Raw": serialization.PrivateFormat.Raw,
        "OpenSSH": serialization.PrivateFormat.OpenSSH,
    }

    publicSerializationMapping = {
        "SubjectPublicKeyInfo": serialization.PublicFormat.SubjectPublicKeyInfo,
        "PKCS1": serialization.PublicFormat.PKCS1,
        "Raw": serialization.PublicFormat.Raw,
        "OpenSSH": serialization.PublicFormat.OpenSSH,
    }

    RSApaddingMapping = {
        "PSS": padding.PSS,
        "OAEP": padding.OAEP,
        "PKCS1v15": padding.PKCS1v15,
    }

    hashMapping = {
        "sha256": hashes.SHA256(),
        "sha384": hashes.SHA384(),
        "sha512": hashes.SHA512(),
    }

    keyUsage = [
        "digital_signature",
        "content_commitment",
        "key_encipherment",
        "data_encipherment",
        "key_agreement",
        "key_cert_sign",
        "crl_sign",
        "encipher_only",
        "decipher_only",
    ]

    extendedKeyUsageMapping = {
        "SERVER_AUTH": x509.oid.ExtendedKeyUsageOID.SERVER_AUTH,
        "CLIENT_AUTH": x509.oid.ExtendedKeyUsageOID.CLIENT_AUTH,
        "CODE_SIGNING": x509.oid.ExtendedKeyUsageOID.CODE_SIGNING,
        "EMAIL_PROTECTION": x509.oid.ExtendedKeyUsageOID.EMAIL_PROTECTION,
        "TIME_STAMPING": x509.oid.ExtendedKeyUsageOID.TIME_STAMPING,
        "OCSP_SIGNING": x509.oid.ExtendedKeyUsageOID.OCSP_SIGNING,
        "ANY_EXTENDED_KEY_USAGE": x509.oid.ExtendedKeyUsageOID.ANY_EXTENDED_KEY_USAGE,
    }



import os
import sys
import datetime
import logging
from ipaddress import IPv4Address, IPv6Address, IPv4Network, IPv6Network

from cryptography import x509

from certsGenerator.conf import Conf
from certsGenerator.helpers import loadFile


class CertBuilder:
    def __init__(
        self,
        certName: str,
        conf: Conf,
        ski: x509.SubjectKeyIdentifier,
    ):
        self.certName = certName
        self.conf: Conf = conf
        self.ski = ski

        self.builder = x509.CertificateBuilder()
        self.certificate: x509.Certificate
        self._setAll()

    def buildSubjectNameAttributes(self, certName: str) -> x509.Name:
        nameAttributesMapping = self.conf.nameAttributesMapping
        nameAttrList = []
        snaConf = self.conf.get(
            certName=certName, field="subject_name_attributes", isExt=False
        )
        for k in snaConf.keys():
            nameAttrList.append(
                x509.NameAttribute(nameAttributesMapping[k], snaConf[k])
            )
        name_attr = x509.Name(nameAttrList)

        return name_attr

    def _setNotValid(self) -> None:
        certConf = self.conf.getCert(certName=self.certName)
        nvb = None
        try:
            if certConf["not_valid_before"] == "now":
                nvb = datetime.datetime.utcnow()
            else:
                nvb = datetime.datetime.utcnow() + datetime.timedelta(
                    days=certConf["not_valid_before"]
                )
        except ValueError() as e:
            logging.error(f"can't set not_valid_before: {e}")
            sys.exit()
        nva: None
        try:
            nva = datetime.datetime.utcnow() + datetime.timedelta(
                days=certConf["not_valid_after"]
            )
        except ValueError() as e:
            logging.error(f"can't set not_valid_after: {e}")
            sys.exit()

        self.builder = self.builder.not_valid_before(nvb)
        self.builder = self.builder.not_valid_after(nva)

    def _setNameAttributes(self) -> None:
        subjectConf = self.conf.getCert(certName=self.certName)
        sna = self.buildSubjectNameAttributes(certName=self.certName)
        self.builder = self.builder.subject_name(sna)
        # set issuer name attributes
        if subjectConf["issuer_name"] == subjectConf["subject_name"]:
            # here we have a self signed certificate (generally the root CA)
            # so name attributes are equal both for issuer and subject
            self.builder = self.builder.issuer_name(sna)
        else:
            # here we need to get the name attributes for the related issuer
            ina = self.buildSubjectNameAttributes(certName=subjectConf["issuer_name"])
            self.builder = self.builder.issuer_name(ina)

    def _setSubjectAlternativeName(self, extConf: dict) -> None:
        items = extConf["items"]
        isCritical = True if extConf["critical"] == "true" else False
        elList = []
        for item in items:
            el = None
            if item.get("DNSName"):
                el = self.conf.extensionMapping["DNSName"](item.get("DNSName"))
            elif item.get("IPAddressV4"):
                el = self.conf.extensionMapping["IPAddress"](
                    IPv4Address(item.get("IPAddressV4"))
                )
            elif item.get("IPAddressV6"):
                el = self.conf.extensionMapping["IPAddress"](
                    IPv6Address(item.get("IPAddressV6"))
                )
            elif item.get("IPNetworkV4"):
                el = self.conf.extensionMapping["IPAddress"](
                    IPv4Network(item.get("IPNetworkV4"))
                )
            elif item.get("IPNetworkV6"):
                el = self.conf.extensionMapping["IPAddress"](
                    IPv6Network(item.get("IPNetworkV6"))
                )
            else:
                logging.error(f"can't find SubjectAlternativeName {item}")
                raise ValueError()
                sys.exit()
            elList.append(el)

        self.builder = self.builder.add_extension(
            x509.SubjectAlternativeName(elList), critical=isCritical
        )

    def _setKeyUsage(self, extConf: dict) -> None:
        kwargs = {}
        for el in self.conf.keyUsage:
            if el in extConf["items"]:
                kwargs[el] = True
            else:
                kwargs[el] = False

        isCritical = True if extConf.get("critical") == "true" else False

        self.builder = self.builder.add_extension(
            x509.KeyUsage(**kwargs), critical=isCritical
        )

    def _setExtendedKeyUsage(self, extConf: dict) -> None:
        eku = []
        for el in extConf["items"]:
            eku.append(self.conf.extendedKeyUsageMapping[el.upper()])

        isCritical = True if extConf.get("critical") == "true" else False

        self.builder = self.builder.add_extension(
            x509.ExtendedKeyUsage(eku), critical=isCritical
        )

    def _setBasicConstraints(self, extConf: dict) -> None:
        pathLenght = None
        if "path_length" in extConf.keys():
            if extConf["path_length"] == "none":
                pathLenght = None
            else:
                pathLenght = int(extConf["path_length"])
        isCA = True if extConf.get("ca") == "true" else False
        isCritical = True if extConf.get("critical") == "true" else False

        self.builder = self.builder.add_extension(
            x509.BasicConstraints(ca=isCA, path_length=pathLenght),
            critical=isCritical,
        )

    def _setNameConstraints(self, extConf: dict) -> None:
        isCritical = True if extConf.get("critical") == "true" else False

        permittedList = []
        if (extConf.get("permitted_subtrees") is not None) and (
            len(extConf["permitted_subtrees"]) >= 1
        ):
            for p in extConf["permitted_subtrees"]:
                permittedList.append(self.conf.extensionMapping[p["type"]](p["value"]))
        else:
            permittedList = None  # type: ignore

        excludedList = []
        if (extConf.get("excluded_subtrees") is not None) and (
            len(extConf["excluded_subtrees"]) >= 1
        ):
            for p in extConf["excluded_subtrees"]:
                excludedList.append(self.conf.extensionMapping[p["type"]](p["value"]))
        else:
            excludedList = None  # type: ignore

        self.builder = self.builder.add_extension(
            x509.NameConstraints(
                permitted_subtrees=permittedList, excluded_subtrees=excludedList
            ),
            critical=isCritical,
        )

    def _setSubjectKeyIdentifier(self, extConf: dict) -> None:
        if extConf["set"] == "true":
            isCritical = True if extConf.get("critical") == "true" else False

            self.builder = self.builder.add_extension(
                self.ski,
                critical=isCritical,
            )

    def _setAuthorityKeyIdentifier(self, extConf: dict) -> None:
        if extConf["set"] == "true":
            # we need to get the SubjectKeyIdentifier of the issuer (aka authority)
            # we'll get it from the public key
            certConf = self.conf.getCert(certName=self.certName)

            if certConf["subject_name"] != certConf["issuer_name"]:
                issuer_name = certConf["issuer_name"]
                issuer_conf = self.conf.getCert(certName=issuer_name)
                issuerCrtFile = self.conf.getCertPath(
                    certName=issuer_name, ext="signed_certificate"
                )
                issuer_cert = None
                # load crt file from constructed path if it exists
                if os.path.exists(issuerCrtFile):
                    cert_data = loadFile(fileName=issuerCrtFile)
                    enc = issuer_conf.get("public_key").get("encoding")
                    if enc == "DER":
                        issuer_cert = x509.load_der_x509_certificate(cert_data)
                    elif enc == "PEM":
                        issuer_cert = x509.load_pem_x509_certificate(cert_data)

                else:
                    logging.error(f"can't find issuer crt file {issuerCrtFile}")
                    raise ValueError()
                    sys.exit()

                isCritical = True if extConf.get("critical") == "true" else False

                self.builder = self.builder.add_extension(
                    x509.AuthorityKeyIdentifier.from_issuer_public_key(
                        issuer_cert.public_key()
                    ),
                    critical=isCritical,
                )
            else:
                logging.error(
                    "AuthorityKeyIdentifier can't be set because subject_name == issuer_name (self-signed), please correct the configuration"
                )
                raise ValueError()
                sys.exit()

    def _setExtensions(self) -> None:
        certConf = self.conf.getCert(certName=self.certName)
        if certConf.get("extensions"):
            extensionsConf = certConf.get("extensions")
            for k in extensionsConf.keys():
                if k == "SubjectAlternativeName":
                    self._setSubjectAlternativeName(extConf=extensionsConf[k])
                elif k == "KeyUsage":
                    self._setKeyUsage(extConf=extensionsConf[k])
                elif k == "ExtendedKeyUsage":
                    self._setExtendedKeyUsage(extConf=extensionsConf[k])
                elif k == "BasicConstraints":
                    self._setBasicConstraints(extConf=extensionsConf[k])
                elif k == "NameConstraints":
                    self._setNameConstraints(extConf=extensionsConf[k])
                elif k == "SubjectKeyIdentifier":
                    self._setSubjectKeyIdentifier(extConf=extensionsConf[k])
                elif k == "AuthorityKeyIdentifier":
                    self._setAuthorityKeyIdentifier(extConf=extensionsConf[k])
                else:
                    logging.error(f"incorrect or not implemented extension {k}")
                    raise ValueError()
                    sys.exit()

    def _setAll(self) -> None:
        # set the cert configuration
        self._setNameAttributes()
        self._setNotValid()
        self._setExtensions()
        self.builder = self.builder.serial_number(x509.random_serial_number())
